
/* document.cc */

#include "document.h"
#include "view.h"

// #include <QtGui/QMdiArea>
// #include <QtGui/QMdiSubWindow>
#include <QtCore/QFile>
#include <QApplication>
#include <KMessageBox>
#include <KLocale>

#include <interfaces/icore.h>
#include <interfaces/iuicontroller.h>
#include <interfaces/idocumentcontroller.h>
#include <sublime/view.h>
#include <sublime/area.h>
#include <sublime/controller.h>

#include "kdevplatformversion.h"

/* ---------------------------------------------------------------------- */

MyDocumentFactory::MyDocumentFactory (MyPlugin* plugin) :
  KDevelop::IDocumentFactory(),
  m_plugin (plugin)
{
}

KDevelop::IDocument* MyDocumentFactory::create (const KUrl& url, KDevelop::ICore* core)
{
   KMimeType::Ptr mimetype = KMimeType::findByUrl (url);
   if (mimetype->name() == "text/x-kview")
   {
      kDebug (8003) << "CREATE DOCUMENT";
      MyDocument* d = new MyDocument (url, core);
      d->setPlugin (m_plugin);
      return d;
   }
   return NULL;
}

/* ---------------------------------------------------------------------- */

MyDocument::MyDocument (const KUrl& url , KDevelop::ICore* core) :
   Sublime::UrlDocument (core->uiController()->controller(), url),
   KDevelop::IDocument (core),
   m_state (KDevelop::IDocument::Clean)
{

}

KSharedPtr<KMimeType> MyDocument::mimeType() const
{
   return KMimeType::mimeType ("application/x-designer");
}

KParts::Part* MyDocument::partForView (QWidget*) const
{
   return 0;
}

KTextEditor::Document* MyDocument::textDocument() const
{
   return 0;
}

bool MyDocument::save (KDevelop::IDocument::DocumentSaveMode mode)
{
   if (mode & Discard)
      return true;

   kDebug (9038) << "Going to Save";
   if (m_state == KDevelop::IDocument::Clean)
      return false;
   // if( !m_form )
   //     return false;
   QFile f (url().toLocalFile());
   if (!f.open (QIODevice::WriteOnly))
   {
      kDebug (9038) << "Couldn't open file:" << f.error();
      return false;
   }
   QTextStream s (&f);
   // s << m_form->contents();
   s.flush();
   f.close();
   m_state = KDevelop::IDocument::Clean;
   notifySaved();
   return true;
}

void MyDocument::reload()
{
   QFile uiFile (url().toLocalFile());
   // m_form->setContents(&uiFile);
   m_state = KDevelop::IDocument::Clean;
   notifyStateChanged();
}

bool MyDocument::close (KDevelop::IDocument::DocumentSaveMode mode)
{
   // kDebug() << "form:" << m_form;
   if (! (mode & Discard))
   {
      if (mode & Silent)
      {
         if (!save (mode))
            return false;
      }
      else
      {
         if (state() == IDocument::Modified)
         {
            int code = KMessageBox::warningYesNoCancel (
                          qApp->activeWindow(),
                          i18n ("The document \"%1\" has unsaved changes. Would you like to save them?", url().toLocalFile()),
                          i18n ("Close Document"));

            if (code == KMessageBox::Yes)
            {
               if (!save (mode))
                  return false;

            }
            else if (code == KMessageBox::Cancel)
            {
               return false;
            }

         }
         else if (state() == IDocument::DirtyAndModified)
         {
            if (!save (mode))
               return false;
         }
      }
   }

   //close all views and then delete ourself
   ///@todo test this
   foreach (Sublime::Area *area,
            KDevelop::ICore::self()->uiController()->controller()->allAreas())
   {
      QList<Sublime::View*> areaViews = area->views();
      foreach (Sublime::View *view, areaViews)
      {
         if (views().contains (view))
         {
            area->removeView (view);
            delete view;
         }
      }
   }

   #if KDEVPLATFORM_VERSION < KDE_MAKE_VERSION(1, 3, 0)
   KDevelop::ICore::self()->documentController()->notifyDocumentClosed (this);

   // Here we go...
   deleteLater();
   #endif

   // The document is deleted automatically when there are no views left

   return true;
}

bool MyDocument::isActive() const
{
   // QDesignerFormWindowInterface* activeWin =
   //         m_designerPlugin->designer()->formWindowManager()->activeFormWindow();
   // if( activeWin == m_form )
   //     return true;
   return false;
}

KDevelop::IDocument::DocumentState MyDocument::state() const
{
   return m_state;
}

void MyDocument::setCursorPosition (const KTextEditor::Cursor&)
{
   return;
}

void MyDocument::setTextSelection (const KTextEditor::Range &)
{
}

void MyDocument::activate (Sublime::View* view, KParts::MainWindow*)
{
   // m_designerPlugin->designer()->formWindowManager()->setActiveFormWindow( m_form );
   notifyActivated();
}

Sublime::View * MyDocument::newView (Sublime::Document* doc)
{
   if (qobject_cast<MyDocument*> (doc))
   {
      QFile uiFile (url().toLocalFile());
      uiFile.open (QIODevice::ReadOnly | QIODevice::Text);
      return new MyViewFactory (m_plugin, doc);
   }
   return 0;
}

/*
void MyDocument::formChanged()
{
    m_state = KDevelop::IDocument::Modified;
    notifyStateChanged();
}
*/

KTextEditor::Cursor MyDocument::cursorPosition() const
{
   return KTextEditor::Cursor();
}

bool MyDocument::closeDocument()
{
   return close();
}


/* ---------------------------------------------------------------------- */

MyViewFactory::MyViewFactory (MyPlugin * plugin, Sublime::Document * doc) :
   Sublime::View (doc),
   m_plugin (plugin)
{ }

MyViewFactory::~MyViewFactory ()
{ }

QWidget* MyViewFactory::createWidget (QWidget* parent)
{
   MyView * v = new MyView (m_plugin, parent);
   v->example ();
   return v;
}

/* ---------------------------------------------------------------------- */

#include "document.moc"

