
/* document.h */

#ifndef MY_DOCUMENT_H
#define MY_DOCUMENT_H

#include <QtCore/QList>

#include <sublime/urldocument.h>
#include <sublime/view.h>
#include <interfaces/idocument.h>
#include <interfaces/idocumentcontroller.h>

/* ---------------------------------------------------------------------- */

namespace KDevelop
{
   class ICore;
}

class MyPlugin;

/* ---------------------------------------------------------------------- */

class MyDocumentFactory : public KDevelop::IDocumentFactory
{
   public:
      MyDocumentFactory (MyPlugin * plugin);
      KDevelop::IDocument* create (const KUrl& url, KDevelop::ICore* core);
   private:
      MyPlugin* m_plugin;
};

/* ---------------------------------------------------------------------- */

class MyDocument : public Sublime::UrlDocument, public KDevelop::IDocument
{
   Q_OBJECT
   public:
      MyDocument (const KUrl&, KDevelop::ICore*);

      void setPlugin (MyPlugin * plugin) { m_plugin = plugin; }

      KUrl url() const { return Sublime::UrlDocument::url(); }

      virtual KSharedPtr<KMimeType> mimeType() const;
      virtual KParts::Part* partForView (QWidget*) const;
      virtual KTextEditor::Document* textDocument() const;
      virtual bool save (KDevelop::IDocument::DocumentSaveMode = KDevelop::IDocument::Default);
      virtual void reload ();
      virtual bool close (KDevelop::IDocument::DocumentSaveMode = KDevelop::IDocument::Default);
      virtual bool isActive () const;
      virtual DocumentState state() const;
      virtual void setCursorPosition (const KTextEditor::Cursor&);
      virtual void setTextSelection (const KTextEditor::Range &range);
      virtual void activate (Sublime::View*, KParts::MainWindow*);
      virtual KTextEditor::Cursor cursorPosition () const;
      virtual bool closeDocument ();

   private Q_SLOTS:
      Sublime::View* newView (Sublime::Document* d);

   private:
      KDevelop::IDocument::DocumentState m_state;
      MyPlugin * m_plugin;
};

/* ---------------------------------------------------------------------- */

class MyViewFactory : public Sublime::View
{
   public:
      MyViewFactory (MyPlugin * plugin, Sublime::Document * doc);
      ~MyViewFactory();
      QWidget* createWidget (QWidget* parent = 0);
   private:
      KDevelop::IDocument::DocumentState m_state;
      MyPlugin * m_plugin;
};

/* ---------------------------------------------------------------------- */

#endif // MY_DOCUMENT_H

