
/* module.cc */

#include "module.h"

#include "document.h"

#include "tree.h"
#include "prop.h"
#include "view.h"

#include <QObject>
#include <QAction>
#include <QFile>
#include <QTextStream>
#include <QPluginLoader>
#include <QWorkspace>
#include <QPushButton>

#include <kaboutdata.h>
#include <kaction.h>
#include <kmimetype.h>
#include <kxmlguiwindow.h>
#include <kpluginfactory.h>
#include <kpluginloader.h>
#include <kparts/mainwindow.h>
#include <kparts/partmanager.h>
#include <ksavefile.h>
#include <kstandardaction.h>
#include <kicon.h>
#include <kactioncollection.h>

#include <interfaces/icore.h>
#include <interfaces/idocumentcontroller.h>
#include <interfaces/iuicontroller.h>

#include <iostream>
// using namespace std;

/* ----------------------------------------------------------------------- */

KAboutData aboutData ("kview", "kview", ki18n ("KView"), "0.1.1",
                       ki18n ("Simple visual designer plugin"),
                       KAboutData::License_GPL);

K_PLUGIN_FACTORY (MyPluginFactory, registerPlugin <MyPlugin> (); )
K_EXPORT_PLUGIN (MyPluginFactory (aboutData))

/* ----------------------------------------------------------------------- */

class ViewFactory : public KDevelop::IToolViewFactory
{
   public:
       virtual QWidget * create (QWidget * parent = 0)
       {
           MyView * view = new MyView (m_plugin, parent);
           view->example ();
           return view;
       }

       virtual Qt::DockWidgetArea defaultPosition()
       {
           return Qt::BottomDockWidgetArea;
       }

       virtual QString id() const
       {
           return "org.kdevelop.KView.Info";
       }

   private:
       MyPlugin * m_plugin;

   public:
       ViewFactory (MyPlugin * plugin) :
           m_plugin (plugin)
       { }
};

/* ----------------------------------------------------------------------- */

class TreeFactory : public KDevelop::IToolViewFactory
{
   public:
       virtual QWidget * create (QWidget * parent = 0)
       {
           return new MyTree (m_plugin, parent);
       }

       virtual Qt::DockWidgetArea defaultPosition()
       {
           return Qt::LeftDockWidgetArea;
       }

       virtual QString id() const
       {
           return "org.kdevelop.KView.Tree";
       }

   private:
       MyPlugin * m_plugin;

   public:
       TreeFactory (MyPlugin * plugin) :
           m_plugin (plugin)
       { }
};

/* ----------------------------------------------------------------------- */

class PropFactory : public KDevelop::IToolViewFactory
{
   public:
       virtual QWidget * create (QWidget * parent = 0)
       {
           return new MyProp (m_plugin, parent);
       }

       virtual Qt::DockWidgetArea defaultPosition()
       {
           return Qt::RightDockWidgetArea;
       }

       virtual QString id() const
       {
           return "org.kdevelop.KView.Prop";
       }

   private:
       MyPlugin * m_plugin;

   public:
       PropFactory (MyPlugin * plugin) :
           m_plugin (plugin)
       { }
};

/* ----------------------------------------------------------------------- */

MyPlugin::MyPlugin (QObject *parent, const QVariantList & args) :
   KDevelop::IPlugin (MyPluginFactory::componentData(), parent),
   m_document_factory (NULL)
{
    Q_UNUSED(args)

    std::cout << std::endl;
    std::cout << " **** KVIEW **** " << std::endl;
    std::cout << std::endl;

    setXMLFile ("kview.rc");

    m_view_factory = new ViewFactory (this);
    core()->uiController()->addToolView ("MyView", m_view_factory);

    m_tree_factory = new TreeFactory (this);
    core()->uiController()->addToolView ("MyTree", m_tree_factory);

    m_prop_factory = new PropFactory (this);
    core()->uiController()->addToolView ("MyProperties", m_prop_factory);

    KActionCollection* actions = actionCollection ();
    KAction *action = actions->addAction ("kview_run");
    action->setText (i18n ("KView Plugin"));
    // action->setShortcut (Qt::CTRL | Qt::ALT | Qt::Key_4);
    connect (action, SIGNAL (triggered (bool)), this, SLOT (run ()));

    #if 1
    kDebug (8003) << "CREATE DOCUMENT FACTORY";
    m_document_factory = new MyDocumentFactory (this);
    KDevelop::IDocumentController* idc = core()->documentController();
    idc->registerDocumentForMimetype("text/x-kview", m_document_factory);
    #endif
}

MyPlugin::~MyPlugin()
{
}

void MyPlugin::unload()
{
    core()->uiController()->removeToolView (m_view_factory);
    core()->uiController()->removeToolView (m_tree_factory);
    core()->uiController()->removeToolView (m_prop_factory);
}

/* ----------------------------------------------------------------------- */

void MyPlugin::run ()
{
    std::cout << std::endl;
    std::cout << " **** KVIEW RUN **** " << std::endl;
    std::cout << std::endl;

    kDebug () << " ---- KVIEW RUN ---- ";
}

/* ----------------------------------------------------------------------- */

#include "module.moc"
